'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.processSVG = processSVG;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _libxmljs = require('libxmljs');

var _libxmljs2 = _interopRequireDefault(_libxmljs);

var _svgo = require('svgo');

var _svgo2 = _interopRequireDefault(_svgo);

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _allRules = require('./rules/all-rules');

var Rules = _interopRequireWildcard(_allRules);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ----------------------------------------------------------------------------
// Public
// ----------------------------------------------------------------------------

/**
 * Callback used to retrieve the sophisticated SVG.
 *
 * @callback processCallback
 * @param {object} result result of the sophistication (SVG)
 */

/**
 * processSVG - Sophisticate the given SVG using the given configuration.
 *
 * @param  {string}          svgPath    path the svg file.
 * @param  {string}          configPath path to the configuration file.
 * @param  {processCallback} callback   handles the completion of the process.
 */
/**
 * @fileoverview Perform SVG sophistication
 * @author Frédéric Maquin <fred@ephread.com>
 */

// ----------------------------------------------------------------------------
// Requirements
// ----------------------------------------------------------------------------

function processSVG(svgPath, configPath, callback) {
  _fs2.default.readFile(svgPath, 'utf-8', function (err, data) {
    if (err) {
      console.log('[' + _chalk2.default.red('ERROR') + '] Couldn\'t read SVG file - ' + err.message);
      process.exit(1);
    }

    var config = loadConfig(configPath);
    var xml = _libxmljs2.default.parseXml(data);
    var svgo = new _svgo2.default(config.svgo);

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = config.sophisticate[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var transformation = _step.value;
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = transformation.rules[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var rule = _step2.value;

            applyTransform(xml, transformation.xpath, rule);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    svgo.optimize(xml.toString(false), callback);
  });
}

// ----------------------------------------------------------------------------
// Private
// ----------------------------------------------------------------------------

/**
 * applyTransform - apply the given tranformation rule on nodes matching the given xpath query.
 *
 * @param  {object} xml   the xml document on which apply the transformation
 * @param  {string} xpath the xpath query selecting the nodes to transform
 * @param  {object} rule  the transformation rule
 */
function applyTransform(xml, xpath, rule) {
  var namespace = 'http://www.w3.org/2000/svg';
  var nodes = xml.find(xpath, namespace);

  if (rule.type in Rules) {
    Rules[rule.type](rule, nodes);
  } else {
    console.log('[' + _chalk2.default.red('ERROR') + '] Rule \'' + rule.type + '\' doesn\'t exist.');
    process.exit(1);
  }
}

/**
 * loadConfig - load the configuration file
 *
 * @param  {string} configPath the path to the configuration file
 * @return {object}            the loaded configuration
 */
function loadConfig(configPath) {
  try {
    var config;

    if (configPath !== undefined) {
      config = _jsYaml2.default.safeLoad(_fs2.default.readFileSync(configPath, 'utf8'));
    } else {
      config = { sophisticate: [] };
    }

    if (config.svgo === undefined) {
      config.svgo = _jsYaml2.default.safeLoad(_fs2.default.readFileSync(__dirname + '/../../config/svgo.yml', 'utf8'));
    }

    return config;
  } catch (err) {
    console.log('[' + _chalk2.default.red('ERROR') + '] Couldn\'t load config file - ' + err.message);
    process.exit(1);
  }
}